package id.device.properties.device_properties
import android.content.Context
import android.content.pm.PackageInfo
import android.content.pm.PackageManager
import android.os.Build
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import kotlinx.coroutines.*
import okhttp3.*
import ru.gildor.coroutines.okhttp.await
import java.io.IOException
import java.util.*
/** DevicePropertiesPlugin */
class DevicePropertiesPlugin : FlutterPlugin, MethodCallHandler {
    private lateinit var channel: MethodChannel
    private lateinit var context: Context
    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "device_properties")
        channel.setMethodCallHandler(this)
        context = flutterPluginBinding.applicationContext
    }
    val scope = CoroutineScope(Dispatchers.IO)
    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        scope.launch {
            when (call.method) {
                "getProperties" -> {
                    result.success(
                        """{"ip_address":"${deviceIpAndroid()}",
                                |"device_type":"${getDeviceName()}",
                                |"os_version":"${Build.VERSION.RELEASE}",
                                |"version_name":"${deviceVersion(context)?.versionName}",
                                |"app_version_code":"${versionCodeAndroid(deviceVersion(context))}"}""".trimMargin()
                    )
                }
                else -> {
                    result.notImplemented()
                }
            }
        }
    }
    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }
}


private fun versionCodeAndroid(packageInfo: PackageInfo?): String {
    return try {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
            "${packageInfo?.longVersionCode}"
        } else {
            "${packageInfo?.versionCode}"
        }
    } catch (e: Exception) {
        "version code not found"
    }
}
private suspend fun deviceIpAndroid(): String? {
    return try {
        getPublicIp().body()?.string()
    } catch (e: Exception) {
        return  "ip address not found"
    }
}

private fun deviceVersion(context: Context): PackageInfo? {
    return try {
        context.packageManager
            .getPackageInfo(context.packageName, 0)
    } catch (e: PackageManager.NameNotFoundException) {
        e.printStackTrace()
        return null
    }
}
suspend fun getPublicIp(): Response {
  try {
      val client = OkHttpClient()
      val request = Request.Builder()
          .url("https://api.ipify.org/")
          .build()
    return client.newCall(request).await()
  }catch (e: IOException){
      throw IOException("ip address not found")
  }
}

private fun getDeviceName(): String {
    val manufacturer = Build.MANUFACTURER
    val model = Build.MODEL
    return if (model.lowercase(Locale.getDefault())
            .startsWith(manufacturer.lowercase(Locale.getDefault()))
    ) {
        model.replaceFirstChar { if (it.isLowerCase()) it.titlecase(Locale.getDefault()) else it.toString() }
    } else {
        manufacturer.replaceFirstChar { if (it.isLowerCase()) it.titlecase(Locale.getDefault()) else it.toString() } + model
    }
}
